import * as pm2 from 'pm2';
/**
 * Handles a message request by returning the requested message.
 */
export declare type MessageHandler = (data?: any) => any;
/**
 * Options for the `getMessages` function.
 */
export interface GetMessagesOptions {
    /**
     * Filter function to select the processes managed by pm2 from which messages need to be requested.
     * Defaults to processes with same name as the active process.
     */
    filter?: (process: pm2.ProcessDescription) => boolean;
    /**
     * Timeout in milliseconds (ms).
     * Defaults to 1000 ms.
     */
    timeout?: number;
}
/**
 * Requests messages from processes managed by pm2.
 */
export declare const getMessages: <T = any>(topic: string, data?: any, { filter, timeout, }?: GetMessagesOptions) => Promise<T[]>;
/**
 * Attaches a message handler for the given topic.
 */
export declare const onMessage: (topic: string, handler: MessageHandler) => void;
declare const _default: {
    getMessages: <T = any>(topic: string, data?: any, { filter, timeout, }?: GetMessagesOptions) => Promise<T[]>;
    onMessage: (topic: string, handler: MessageHandler) => void;
};
export default _default;
