"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert = require("assert");
const pm2 = require("pm2");
;
;
/**
 * Attached message handles.
 */
const handlers = {};
/**
 * The pid of the current process for the pm2 God daemon process.
 */
const myPmId = Number(process.env.pm_id);
/**
 * The name of the current process given in the original start command.
 */
const myName = process.env.name;
/**
 * Handles a request packet.
 */
process.on('message', ({ topic, data: { targetInstanceId, data } }) => __awaiter(this, void 0, void 0, function* () {
    if (typeof handlers[topic] === 'function' && process.send) {
        const response = {
            type: `process:${targetInstanceId}`,
            data: yield handlers[topic](data),
        };
        process.send(response);
    }
}));
/**
 * Requests messages from processes managed by pm2.
 */
exports.getMessages = function getMessages(topic, data, { filter = (process) => process.name === myName, timeout = 1000, } = {}) {
    assert.equal(typeof handlers[topic], 'function', `Handler for ${topic} not attached or not a function`);
    return new Promise((resolve, reject) => {
        const timer = setTimeout(() => reject(new Error(`${topic} timed out`)), timeout);
        const done = function done(err, messages) {
            clearTimeout(timer);
            if (err)
                reject(err);
            else
                resolve(messages);
        };
        pm2.connect(false, (err) => {
            if (err)
                return done(err, []);
            new Promise((resolve, reject) => {
                pm2.list((err, processes) => {
                    if (err)
                        return reject(err);
                    const targets = [];
                    const messages = [];
                    const promises = [];
                    for (const process of processes) {
                        if (!filter(process))
                            continue;
                        if (process.pm_id === myPmId) {
                            promises.push(Promise.resolve()
                                .then(() => handlers[topic](data))
                                .then((message) => { messages.push(message); }));
                        }
                        else if (typeof process.pm_id === 'number') {
                            targets.push(process.pm_id);
                        }
                    }
                    if (targets.length) {
                        promises.push(new Promise((resolve, reject) => {
                            pm2.launchBus((err, bus) => {
                                if (err)
                                    return reject(err);
                                let pending = targets.length;
                                bus.on(`process:${myPmId}`, ({ data }) => {
                                    messages.push(data);
                                    pending -= 1;
                                    if (!pending)
                                        resolve();
                                });
                                const request = { topic, data: { targetInstanceId: myPmId, data } };
                                targets.forEach((pmId) => {
                                    pm2.sendDataToProcessId(pmId, request, (err) => err && reject(err));
                                });
                            });
                        }));
                    }
                    Promise.all(promises)
                        .then(() => resolve(messages))
                        .catch(reject);
                });
            })
                .then((messages) => done(null, messages))
                .catch((err) => done(err, []))
                .finally(() => pm2.disconnect());
        });
    });
};
/**
 * Attaches a message handler for the given topic.
 */
exports.onMessage = function onMessage(topic, handler) {
    handlers[topic] = handler;
};
exports.default = { getMessages: exports.getMessages, onMessage: exports.onMessage };
//# sourceMappingURL=index.js.map